	.NOLIST
	.DEVICE	ATTiny13
	.INCLUDE	"TN13DEF.INC"
	.INCLUDE "\jobs\avr\BSIMACROS.INC"
	.LIST
	.LISTMAC

;--------Equates.----------------------------

	.EQU	CH1	= PINB4
	.EQU	CH0	= PINB3
	.EQU	OWIO	= PINB2
	.EQU	DIR	= PINB1
	.EQU	PULSE	= PINB0
	.EQU	VERSION	= 1

;--------Register Definitions----------------

	.DEF	RCVBUF	= R1
	.DEF	OWSHIFT	= R2
	.DEF	SSAVE	= R3
	.DEF	ACCX	= R4
	.DEF	pCOUNTL	= R5
	.DEF	pCOUNTH	= R6

	.DEF	MACREG	= R16
	.DEF	ACC	= R17
	.DEF	RSTTMR	= R18
	.DEF	TIMER	= R19
	.DEF	BITCNT	= R20

	.DEF	IACC	= R22
	.DEF	ENCFLGS	= R23
	.DEF	IOFLGS	= R24
	.DEF	PBSAVE	= R25
	.DEF	COUNTL	= R26
	.DEF	COUNTH	= R27
	
	.EQU	BITTMR	= 7		;IOFLGS bits
	.EQU	RBFULL	= 6
	.EQU	XMTNG	= 5
	.EQU	XMTZERO	= 4
	.EQU	RCVONE	= 3

	.EQU	COUNTING	= 7		;ENCFLGS bits.
	.EQU	SAMPLE1	= 6
	.EQU	SAMPLE0	= 5
	.EQU	DIFF	= 4		

;--------Code.-------------------------------

	.CSEG

	.ORG	0
	CLR	R0
	RJMP	START
	RJMP	PCISR
	RJMP	TMRISR

;--------Start here.-------------------------	

START:	OUTI	TCCR0B,0b00000001		;Set clock for internal, no prescaler.
	OUTI	PORTB, 0b00000000		;Clear port B.
	OUTI	DDRB,  0b00000000		;All pins are inputs.
	OUTI	PCMSK, 0b00001100		;Interrupt on OWIO and CH0 only.
	OUTI	GIMSK, 0b00100000		;Enable interrupt on pin change.
	OUTI	TIMSK0,0b00000010		;Enable TMR0 overflow interrupt.
	LDI	ZL,0x60			;Point Z to beginning of RAM.
	CLR	ZH			;(ZH should always be zero.)

CLRLP:	ST	Z,R0			;Clear RAM & Reg locations, but not I/O space.
	INC	ZL			;Point to next. (No Z+ for no carry after 0xFF.)
	CPI	ZL,0x1E			;ZL points to ZL?
	BRNE	CLRLP			;  No:  Branch back.

	LDI	PBSAVE,PINB

RESET:	CLI
	OUTI	SPL,RAMEND
	ANDI	ENCFLGS,1<<COUNTING
	SEI

MAINLP:	RCALL	GETBYTE
TSTCMD:	ORI	ACC,0x20
	CPI	ACC,'r'
	BRNE	TRYDIF
	CLRB	ENCFLGS,DIFF
	RJMP	READLP

TRYDIF:	CPI	ACC,'d'
	BRNE	TRYCLR
	SETB	ENCFLGS,DIFF

READLP:	TST	BITCNT
	BRNE	READLP

	SETB	ENCFLGS,SAMPLE0
	RCALL	PUTBYTE
	RCALL	PUTBYTE
	RJMP	READLP

TRYCLR:	CPI	ACC,'z'
	BRNE	TRYSET

	CLR	ACC
	CLR	ACCX
	RJMP	SETIT

TRYSET:	CPI	ACC,'w'
	BRNE	TRYGO

	RCALL	GETBYTE
	MOV	ACCX,ACC
	RCALL	GETBYTE

SETIT:	CLI
	MOV	COUNTL,ACCX
	MOV	COUNTH,ACC
	SEI
	RJMP	MAINLP

TRYGO:	CPI	ACC,'g'
	BRNE	TRYSTP

	SETB	ENCFLGS,COUNTING
	RJMP	MAINLP

TRYSTP:	CPI	ACC,'s'
	BRNE	TRYVER

	CLRB	ENCFLGS,COUNTING
	RJMP	MAINLP

TRYVER:	CPI	ACC,'v'
	BRNE	MAINLP

	LDI	ACC,'E'
	RCALL	PUTBYTE
	LDI	ACC,'N'
	RCALL	PUTBYTE
	LDI	ACC,VERSION
	RCALL	PUTBYTE
	RJMP	MAINLP


;--------I/O routines------------------------

GETBYTE:	JBRC	IOFLGS,RBFULL,GETBYTE	;Read byte: Loop until there's a byte to read.
	MOV	ACC,RCVBUF		;Get it.
	CLRB	IOFLGS,RBFULL		;Clear the flag.
	RET				;Return.

PUTBYTE:	TST	BITCNT			;Send byte: Is bit count zero?
	BRNE	PUTBYTE			;  No:  Loop until it is.

	CLI				;  Yes: Disable interrupts.
	MOV	OWSHIFT,ACC		;       Copy ACC to shift register.
	SETB	IOFLGS,XMTNG		;       Set transmit flag.
	LDI	BITCNT,8			;       Initialize bit count.
	SEI				;       Reenable interrupts.
	RET				;       Return.

;--------Pin change ISR----------------------

PCISR:	IN	SSAVE,SREG		;Save status in SSAVE. No need to push.
	IN	IACC,PINB
	EOR	IACC,PBSAVE
	EOR	PBSAVE,IACC

TRYB:	JBRC	ENCFLGS,COUNTING,TRYOW

	JBRC	IACC,CH0,TRYOW

	JBRC	PBSAVE,CH1,TRYOW

	SBRC	PBSAVE,CH0
	ADIW	COUNTL,1
	SBRS	PBSAVE,CH0
	SBIW	COUNTL,1

TRYOW:	JBRC	IACC,OWIO,ISROUT

	JBRS	PBSAVE,OWIO,OWIOHI	;OWIO Low->High?

OWIOLO:	CLR	RSTTMR			;  No:  High->Low. Clear the reset timer.

	JBRC	IOFLGS,XMTNG,RCVLO	;       Transmitting?

	CPI	BITCNT,8
	BRNE	XMTLO

	JBRC	ENCFLGS,SAMPLE0,XMTLO

	CLRB	ENCFLGS,SAMPLE0
	MOV	OWSHIFT,COUNTL
	MOV	ACC,COUNTH
	JBRC	ENCFLGS,DIFF,SAVECNT

	SUB	OWSHIFT,pCOUNTL
	SBC	ACC,pCOUNTH

SAVECNT:	MOV	pCOUNTL,COUNTL
	MOV	pCOUNTH,COUNTH

XMTLO:	LSR	OWSHIFT			;         Yes: Shift next bit into carry. Is it a zero?
	BRCS	ISROUT			;                No:  We're done.

	SBI	DDRB,OWIO		;                Yes: Assert pin low.
	ORI	IOFLGS,(1<<BITTMR)|(1<<XMTZERO) ;                Set bit timer and transmit zero flag.
	RJMP	ISROUT			;                     We're done.

RCVLO:	ORI	IOFLGS,(1<<BITTMR)|(1<<RCVONE) ;     No:  Set bit timer and, conditionally, "receive one".
	TST	BITCNT			;              Starting a new byte?
	BRNE	ISROUT			;                No:  Done.

	LDI	BITCNT,8			;                Yes: Initialize bit count.
	RJMP	ISROUT			;                     Done.

OWIOHI:	CPI	RSTTMR,10		;  Yes: Was low more than 260uSec?
	BRCC	SETRST			;         Yes: Go setup reset.

	JBRS	IOFLGS,XMTNG,XMTHI	;       Transmitting?

RCVHI:	LSR	OWSHIFT			;         No:  Receiving. Shift to make room for next bit.
	BST	IOFLGS,RCVONE		;              Bit is RCVONE.
	BLD	OWSHIFT,7		;              Transfer it to OWSHIFT.7
	DEC	BITCNT			;              Decrement the bit count.
	BRNE	ISROUT			;              Is it now zero?
	
	MOV	RCVBUF,OWSHIFT		;                Yes: Transfer shift register to holding buffer.
	SETB	IOFLGS,RBFULL		;                     Set full flag.
	RJMP	ISROUT			;                     Done.

XMTHI:	DEC	BITCNT			;         Yes: Decrement bit count.
	BRNE	ISROUT			;              Is it zero?

	CLRB	IOFLGS,XMTNG		;                Yes: Done transmitting.
	RJMP	ISROUT			;                     Done.

SETRST:	RJMP	RESET			;Detected a reset; go do it.

;--------Timer ISR---------------------------

TMRISR:	IN	SSAVE,SREG		;Save status,
	INC	RSTTMR			;Increment reset timer.
	SUBI	IOFLGS,1<<BITTMR		;Decrement bit timer.
	BRCC	ISROUT			;Did it underflow?
	
	CLRB	IOFLGS,RCVONE		;  Yes: We've received (or transmitted) a zero.
	JBRC	IOFLGS,XMTZERO,ISROUT	;       Transmitting?

	ANDI	IOFLGS,255-(1<<XMTZERO)-(1<<BITTMR); Yes: Clear transmit and bit timer flags.
	CBI	DDRB,OWIO		;              Release I/O pin.

ISROUT:	OUT	SREG,SSAVE
	RETI				;And out.
